@@ -1,4 +1,5 @@
 import sys
+from time import strftime
 from django.db import models
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
@@ -12,6 +13,7 @@ from django.core.files.storage import get_storage_class
 from django.template.loader import get_template, select_template
 from django.template import Template, Context, TemplateDoesNotExist
 from django.core.exceptions import ImproperlyConfigured
+from django.utils.translation import ugettext as _
 
 from massmedia import settings as appsettings
 from fields import Metadata, SerializedObjectField, MetadataJSONEncoder, MetadataJSONDecoder
@@ -66,6 +68,44 @@ DOC_STORAGE  = get_storage_class(appsettings.DOC_STORAGE)
 is_image = lambda s: os.path.splitext(s)[1][1:] in appsettings.IMAGE_EXTS
 value_or_list = lambda x: len(x) == 1 and x[0] or x
 
+def super_force_ascii(bad_string):
+    """
+    For unicode strings that are improperly encoded, 1. convert to latin-1 to 
+    make it a regular string, convert it back to a unicode string, assuming that
+    the string is encoded using default windows encoding. Then return an ascii
+    string using xmlcharrefreplace for oddball characters
+    """
+    output = u''
+    for char in bad_string:
+        try:
+            if ord(char) > 127:
+                if isinstance(char, unicode):
+                    bs1 = char.encode('latin-1', 'ignore')
+                else:
+                    bs1 = char
+                bs2 = bs1.decode('cp1252', 'ignore')
+                output = u"%s%s" % (output, bs2)
+            else:
+                output = u"%s%s" % (output, char)
+        except UnicodeDecodeError:
+            continue
+    return output.encode('ascii', 'xmlcharrefreplace')
+
+def custom_upload_to(prefix_path):
+    """ Clean the initial file name and build a destination path based on settings as prefix_path"""
+    def upload_callback(instance, filename):
+        #-- Split and clean the filename with slugify
+        filename = os.path.basename(filename)
+        name, dot, extension = filename.rpartition('.')
+        slug = slugify(name)
+        clean_filename = '%s.%s' % (slug, extension.lower())
+        #-- Build a destination path with previous cleaned string.
+        destination_path = os.path.join(strftime(prefix_path), clean_filename)
+
+        return destination_path
+
+    return upload_callback
+
 class PublicMediaManager(CurrentSiteManager):
     def __init__(self):
         super(PublicMediaManager, self).__init__('site')
@@ -88,15 +128,15 @@ class Media(models.Model):
     site = models.ForeignKey(Site, related_name='%(class)s_site')
     if HAS_TAGGING:
         categories = TagField(blank=True,null=True)
-    reproduction_allowed = models.BooleanField("we have reproduction rights for this media", default=True)
-    public = models.BooleanField(help_text="this media is publicly available", default=True)
-    external_url = models.URLField(blank=True,null=True,help_text="If this URLField is set, the media will be pulled externally")
+    reproduction_allowed = models.BooleanField(_("we have reproduction rights for this media"), default=True)
+    public = models.BooleanField(help_text=_("this media is publicly available"), default=True)
+    external_url = models.URLField(blank=True,null=True,help_text=_("If this URLField is set, the media will be pulled externally"))
     mime_type = models.CharField(max_length=150,blank=True,null=True)
-    width = models.IntegerField(blank=True, null=True, help_text="The width of the widget for the media")
-    height = models.IntegerField(blank=True, null=True, help_text="The height of the widget for the media")
+    width = models.IntegerField(blank=True, null=True, help_text=_("The width of the widget for the media"))
+    height = models.IntegerField(blank=True, null=True, help_text=_("The height of the widget for the media"))
     
     widget_template = models.CharField(max_length=255,blank=True,null=True,
-                help_text='The template name used to generate the widget (defaults to mime_type layout)')
+                help_text=_("The template name used to generate the widget (defaults to mime_type layout)"))
     
     objects = PublicMediaManager()
     
@@ -138,9 +178,9 @@ class Media(models.Model):
         super(Media, self).save(*args, **kwargs)
     
     def thumb(self):
-        return '<p>No Thumbnail Available</p>'
+        return "<p>" + _("No Thumbnail Available") + "</p>"
     thumb.allow_tags = True
-    thumb.short_description = 'Thumbnail'
+    thumb.short_description = _("Thumbnail")
     
     def get_mime_type(self):
         if self.mime_type:
@@ -176,7 +216,7 @@ class Media(models.Model):
                 try:
                     return select_template(lookups)
                 except TemplateDoesNotExist, e:
-                    raise TemplateDoesNotExist("Can't find a template to render the media. Looking in %s" % ", ".join(lookups))
+                    raise TemplateDoesNotExist(_("Can't find a template to render the media. Looking in %s") % ", ".join(lookups))
             else:
                 lookups = [
                     dict(mimetype=mime_type, name=template_type),
@@ -219,15 +259,17 @@ class Media(models.Model):
         except (InputStreamError, HachoirError):
             return
         data = dict([(x.description, value_or_list([item.value for item in x.values])) for x in sorted(metadata) if x.values])
-        for key, val in data.items():
-            if isinstance(val, basestring):
-                data[key] = val.encode('latin-1', 'ignore').decode('cp1252')
         
         if is_image(path) and iptc:
             try:
                 data.update(IPTCInfo(path).__dict__['_data'])
             except:
                 pass
+        
+        for key, val in data.items():
+            if isinstance(val, basestring):
+                data[key] = super_force_ascii(val)
+        
         self.metadata = Metadata(data)
 
 class Image(Media):
@@ -236,14 +278,14 @@ class Image(Media):
     cause a problem if the file doesn't exist and you merely access the record.
     """
     file = models.FileField( #ImageField(
-        upload_to = appsettings.IMAGE_UPLOAD_TO,
+        upload_to =  custom_upload_to(appsettings.IMAGE_UPLOAD_TO),
         blank = True, 
         null = True,
         #width_field='width',
         #height_field='height',
         storage=IMAGE_STORAGE())
     thumbnail = models.ImageField(
-        upload_to = appsettings.THUMB_UPLOAD_TO,
+        upload_to = custom_upload_to(appsettings.THUMB_UPLOAD_TO),
         blank = True,
         null = True,
         width_field = 'thumb_width',
@@ -319,10 +361,11 @@ class Image(Media):
         tags.extend(self.metadata["15"] or [])
         tags.extend(self.metadata["20"] or [])
         tags.extend(self.metadata["25"] or [])
-        self.categories = ", ".join([x[:50] for x in tags])
+        categories = ", ".join([x[:50] for x in tags])
+        self.categories = super_force_ascii(categories)
 
 class Embed(Media):
-    code = models.TextField(help_text='Embed HTML source code',blank=True,null=True)
+    code = models.TextField(help_text=_("Embed HTML source code"),blank=True,null=True)
 
     @property
     def media_url(self):
@@ -330,7 +373,7 @@ class Embed(Media):
 
 class Video(Media):
     file = models.FileField(
-        upload_to = appsettings.VIDEO_UPLOAD_TO,
+        upload_to = custom_upload_to(appsettings.VIDEO_UPLOAD_TO),
         blank = True, 
         null = True,
         storage=VIDEO_STORAGE())
@@ -342,8 +385,7 @@ class Video(Media):
         else:
             return ''
     thumb.allow_tags = True
-    thumb.short_description = 'Thumbnail'
-    
+    thumb.short_description = _("Thumbnail")    
     @property
     def media_url(self):
         return self.external_url or self.file.url
@@ -355,8 +397,8 @@ class Video(Media):
 
 
 class GrabVideo(Video):
-    asset_id = models.CharField(max_length=255,help_text='Grab video asset ID (the `a` parameter)')
-    layout_id = models.CharField(max_length=255,help_text='Grab video asset ID (the `m` parameter)')
+    asset_id = models.CharField(max_length=255,help_text=_("Grab video asset ID (the `a` parameter)"))
+    layout_id = models.CharField(max_length=255,help_text=_("Grab video asset ID (the `m` parameter)"))
     
     if HAS_TAGGING:
         keywords = TagField(null=True,blank=True)
@@ -369,14 +411,14 @@ class GrabVideo(Video):
 
 class Audio(Media):
     file = models.FileField(
-        upload_to = appsettings.AUDIO_UPLOAD_TO,
+        upload_to = custom_upload_to(appsettings.AUDIO_UPLOAD_TO),
         blank = True, 
         null = True,
         storage=AUDIO_STORAGE())
     
     class Meta:
-        verbose_name="audio clip"
-        verbose_name_plural="audio clips"
+        verbose_name=_("audio clip")
+        verbose_name_plural=_("audio clips")
     
     @property
     def media_url(self):
@@ -384,15 +426,15 @@ class Audio(Media):
 
 class Flash(Media):
     file = models.FileField(
-        upload_to = appsettings.FLASH_UPLOAD_TO,
+        upload_to = custom_upload_to(appsettings.FLASH_UPLOAD_TO),
         blank = True, 
         null = True,
         storage=FLASH_STORAGE())
     
     
     class Meta:
-        verbose_name="SWF File"
-        verbose_name_plural="SWF Files"
+        verbose_name=_("SWF File")
+        verbose_name_plural=_("SWF Files")
     
     @property
     def media_url(self):
@@ -400,11 +442,15 @@ class Flash(Media):
     
 class Document(Media):
     file = models.FileField(
-        upload_to = appsettings.DOC_UPLOAD_TO,
+        upload_to = custom_upload_to(appsettings.DOC_UPLOAD_TO),
         blank = True, 
         null = True,
         storage=DOC_STORAGE())
     
+    class Meta:
+        verbose_name=_("Document")
+        verbose_name_plural=_("Documents")
+    
     @property
     def media_url(self):
         return self.external_url or self.file.url
@@ -414,9 +460,9 @@ class Collection(models.Model):
     title = models.CharField(max_length=255)
     slug = models.SlugField(unique=True)
     caption = models.TextField(blank=True)
-    zip_file = models.FileField('Media files in a .zip', upload_to='tmp', blank=True,null=True,
-                        help_text='Select a .zip file of media to upload into a the Collection.')
-    public = models.BooleanField(help_text="this collection is publicly available", default=True)
+    zip_file = models.FileField(_("Media files in a .zip"), upload_to='tmp', blank=True,null=True,
+                        help_text=_("Select a .zip file of media to upload into a the Collection."))
+    public = models.BooleanField(help_text=_("this collection is publicly available"), default=True)
     site = models.ForeignKey(Site)
     
     if HAS_TAGGING:
@@ -443,7 +489,7 @@ class Collection(models.Model):
         if self.zip_file and os.path.isfile(self.zip_file.path):
             zip = zipfile.ZipFile(self.zip_file.path)
             if zip.testzip():
-                raise Exception('"%s" in the .zip archive is corrupt.' % zip)
+                raise Exception(_('"%s" in the .zip archive is corrupt.') % zip)
             for filename in zip.namelist():
                 if filename.startswith('__') or filename.startswith('.'):
                     # do not process hidden or meta files
@@ -489,12 +535,16 @@ class Collection(models.Model):
                 pass
             super(Collection, self).save(*(), **{})
 
-collection_limits = {'model__in':('image','audio','video','flash')}
+collection_limits = {'model__in':('image','audio','video','document','flash')}
 class CollectionRelation(models.Model):
     collection = models.ForeignKey(Collection)
     content_type = models.ForeignKey(ContentType, limit_choices_to=collection_limits)
     object_id = models.PositiveIntegerField()
     content_object = generic.GenericForeignKey('content_type', 'object_id')
+    position = models.PositiveSmallIntegerField(_("Position"), default = 0, blank = True, null=True, editable=True)
+    
+    class Meta:
+        ordering = ['position', 'id']
     
     def __unicode__(self):
         return unicode(self.content_object)
@@ -509,3 +559,6 @@ class MediaTemplate(models.Model):
     
     def template(self):
         return Template(self.content)
+        
+        
+
